using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using JetBrains.ReSharper.ContextActions.CSharp;
using JetBrains.ReSharper.Daemon;
using JetBrains.ReSharper.Editor;
using JetBrains.ReSharper.ProjectModel;
using JetBrains.ReSharper.Psi;
using JetBrains.ReSharper.Psi.CSharp;
using JetBrains.ReSharper.Psi.CSharp.CodeStyle;
using JetBrains.ReSharper.TextControl;
using JetBrains.ReSharper.TextControl.Markup;
using JetBrains.ReSharper.UsageHightlighting.Impl;
using JetBrains.ReSharper.Util;

namespace AccessibilityAnalayzer
{
    [CSharpContextAction]
    public class AccessibilityContextAction:ContextActionBase 
    {
        private AccessibilitySuggestion accessibilitySuggestion;
        private IDeclaration declaration;
        private IDeclaredElement declaredElement;
        IHighlighter myHighlighter;
        IBulbItem[] bulbItems;
        
        public AccessibilityContextAction(ICSharpContextActionDataProvider provider) : base(provider)
        {
            
        }

        public override IBulbItem[] Items
        {
            get { return bulbItems; }
        }
        
       protected override void ExecuteInternal(params object[] param)
        {
            switch (accessibilitySuggestion)
            {
                case AccessibilitySuggestion.RemoveUnused:
                    if ((param[0] as string).StartsWith("Comment"))
                        ExecuteCommentUnused();
                    else
                        ExectuteRemoveUnused();
                    RemoveHighligher();
                    break;
                case AccessibilitySuggestion.CanBePrivate:
                    ChangeModifier(AccessRights.PRIVATE);
                    RemoveHighligher();
                    break;
                case AccessibilitySuggestion.CanBeInternal:
                    ChangeModifier(AccessRights.INTERNAL);
                    RemoveHighligher();
                    break;
                case AccessibilitySuggestion.OK:
                    break;
                default:
                    break;
            }
        }

        private void ChangeModifier(AccessRights accessRights)
        {
            IModifiersListOwnerNode modifiersListOwnerNode=declaredElement as IModifiersListOwnerNode;
            if(modifiersListOwnerNode!=null)
            {
                AccessUtil.SetAccessRights(modifiersListOwnerNode, accessRights);
            }
        }

        private void ExectuteRemoveUnused()
        {
            ICSharpDeclaration declaration1 = declaration as ICSharpDeclaration;
            if (declaration1 is IClassMemberDeclaration)
            {
                IClassLikeDeclaration declaration2 = declaration1.GetContainingTypeDeclaration() as IClassLikeDeclaration;
                if (declaration2 != null)
                {
                    declaration2.RemoveClassMemberDeclaration((IClassMemberDeclaration)declaration1);
                }
            }
            else if (declaration1 is IEnumMemberDeclaration)
            {
                IEnumDeclaration declaration3 = declaration1.GetContainingTypeDeclaration() as IEnumDeclaration;
                if (declaration3 != null)
                {
                    declaration3.RemoveEnumMemberDeclaration((IEnumMemberDeclaration)declaration1);
                }
            }
            else if (declaration1 is IAccessorDeclaration)
            {
                ITypeMemberDeclaration declaration4 = declaration1.GetContainingTypeMemberDeclaration();
                if (declaration4 is IPropertyDeclaration)
                {
                    ((IPropertyDeclaration)declaration4).RemoveAccessorDeclaration((IAccessorDeclaration)declaration1);
                }
                else if (declaration4 is IIndexerDeclaration)
                {
                    ((IIndexerDeclaration)declaration4).RemoveAccessorDeclaration((IAccessorDeclaration)declaration1);
                }
                else if (declaration4 is IEventDeclaration)
                {
                    ((IEventDeclaration)declaration4).RemoveAccessorDeclaration((IAccessorDeclaration)declaration1);
                }
                else
                {
                    Logger.Assert(false, "typeMemberDeclaration is {0}", new object[] { declaration4.ToString() });
                }
            }
            else
            {
                Logger.Assert(false, "declaration is {0}", new object[] { declaration1.ToString() });
            }

        }
        
        private void ExecuteCommentUnused()
        {
            TextRange range = declaration.GetDocumentRange().TextRange;
            IDocument document1 = Document;
            IFile file = this.declaration.GetContainingFile();
            DocumentCoords coords1 = document1.GetCoordsByOffset(range.StartOffset);
            int num1 = coords1.Line;
            DocumentCoords coords2 = document1.GetCoordsByOffset(range.EndOffset);
            int num2 = coords2.Line + 1;
            TextRange range1 = new TextRange(document1.GetLineStartOffset(num1), document1.GetLineStartOffset(num2));
            ITokenNode node1 = file.FindTokenAt(range1.StartOffset) as ITokenNode;
            ITokenNode node2 = file.FindTokenAt(range1.EndOffset - 1) as ITokenNode;
            bool flag1 = true;
            //while ((node1 != node2) && (node1 != null))
            //{
            //    ICSharpCommentNode node3 = node1 as ICSharpCommentNode;
            //    if ((node3 != null) && (node3.CommentType == CommentType.MULTILINE_COMMENT))
            //    {
            //        flag1 = true;
            //        break;
            //    }
            //    node1 = node1.GetNextToken();
            //}
            if (flag1)
            {
                for (int num3 = num2 - 1; num3 >= num1; num3--)
                {
                    file = file.ReParse(new TextRange(document1.GetLineStartOffset(num3)), "//");
                }
            }
            else
            {
                string text1 = CSharpCodeStyleSettings.GetInstance(file.GetManager().Solution).FormatSettings.NewLine;
                string text2 = "/*" + text1;
                string text3 = "*/" + text1;
                file = file.ReParse(new TextRange(range1.StartOffset), text2);
                file = file.ReParse(new TextRange(range1.EndOffset + text2.Length), text3);
            }

        }

        protected override bool IsAvailableInternal()
        {
            ArrayList list = this.Document.GetData(AccessibilityHighlightingManager.HIGHLIGHTERS_KEY) as ArrayList;
            if (list == null)
                return false;
            foreach (IHighlighter highlighter in list)
            {
                if (highlighter.Range.Intersects(this.SelectedElement.GetDocumentRange().TextRange))
                {
                    if (highlighter.UserData is AccessibilityHighlightEntryUserData)
                    {
                        AccessibilityHighlightEntryUserData accessibilityHighlightEntryUserData = (AccessibilityHighlightEntryUserData) highlighter.UserData;
                        this.accessibilitySuggestion = accessibilityHighlightEntryUserData.AccessibilitySuggestion;
                        this.declaration = accessibilityHighlightEntryUserData.Declaration;
                        declaredElement = accessibilityHighlightEntryUserData.DeclaredElement;
                        myHighlighter = highlighter;
                        CreateBulbItem();
                        return true;
                    } 
                    
                }
            }
            return false;
        }

        private void CreateBulbItem()
        {
            List<BulbItem> bulbItemsList = new List<BulbItem>();

            switch (accessibilitySuggestion)
            {
                case AccessibilitySuggestion.RemoveUnused:
                    bulbItemsList.Add(new BulbItem("Remove unused member", this));
                    //bulbItemsList.Add(new BulbItem("Comment unused member", this));
                    break;
                case AccessibilitySuggestion.CanBePrivate:
                    bulbItemsList.Add(new BulbItem("Reduce accessibility to private", this));
                    break;
                case AccessibilitySuggestion.CanBeInternal:
                    bulbItemsList.Add(new BulbItem("Reduce accessibility to internal", this));
                    break;
            }
            this.bulbItems = bulbItemsList.ToArray();
        }
        
        private void RemoveHighligher()
        {
            ArrayList list = this.Document.GetData(AccessibilityHighlightingManager.HIGHLIGHTERS_KEY) as ArrayList;
            if (list == null)
                return;
            this.TextControl.MarkupModel.RemoveHighlighter(myHighlighter);
            list.Remove(myHighlighter);
           
        }

        
        internal class BulbItem:IBulbItem
        {
            private string text;
            private readonly ContextActionBase context;

            public BulbItem(string text,ContextActionBase context)
            {
                this.text = text;
                this.context = context;
            }

            public void Execute(ISolution solution, ITextControl textControl)
            {
                context.Execute(solution, textControl, Text, new object[] { this.Text });
            }

            public string Text
            {
                get { return text; }
            }
        }
       
    }
    
    
}
