using System;
using System.Collections.Generic;
using System.Text;
using JetBrains.ReSharper;
using JetBrains.ReSharper.ActionManagement;
using JetBrains.ReSharper.ProjectModel;
using JetBrains.ReSharper.Psi;
using JetBrains.ReSharper.Shell;
using JetBrains.ReSharper.TextControl;
using JetBrains.ReSharper.UI.PopupWindowManager;
using JetBrains.ReSharper.UsageHightlighting;
using JetBrains.ReSharper.UsageHightlighting.Impl;

namespace AccessibilityAnalayzer
{
    [ActionHandler("AccessibilityAnalayzer.AnalyzeAccessibilityAction")]
    internal class AnalyzeAccessibilityAction : IActionHandler
    {
        public bool Update(IDataContext context, ActionPresentation presentation, DelegateUpdate nextUpdate)
        {
             return context.CheckAllNotNull(new IDataConstant[] { DataConstants.TEXT_CONTROL, DataConstants.POPUP_WINDOW_CONTEXT });
        }

        public void Execute(IDataContext context, DelegateExecute nextExecute)
        {
            ITextControl control1 = (ITextControl)context.GetData(DataConstants.TEXT_CONTROL);
            ISolution solution1 = (ISolution)context.GetData(DataConstants.SOLUTION);
            IProjectModelElement projectModelElement = (IProjectModelElement)context.GetData(DataConstants.PROJECT_MODEL_ELEMENT);
            if (PsiManager.GetInstance(solution1).WaitForCaches())
            {
                IPopupWindowContext context1 = (IPopupWindowContext)context.GetData(DataConstants.POPUP_WINDOW_CONTEXT);
                if ((solution1 != null) && (context1 != null))
                {
                    IHighlightingRangeEvaluator evaluator1 = AccessibilityHighlightingRangeEvaluatorFactory.CreateHighlightingRangeEvaluator(solution1, control1, projectModelElement);
                    if (evaluator1 != null)
                    {
                        UsageHighlightingEntry[] entryArray1 = evaluator1.Entries;
                        if (entryArray1.Length > 0)
                        {
                            AccessibilityHighlightingManager.Instance.Highlight(control1, entryArray1, false);
                        }
                    }
                }
            }


           
        }
    }
}
